/*
 * ExtensionBuilder.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso;


import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.Extension;
import org.jabberstudio.jso.StreamContext;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamNode;
import org.jabberstudio.jso.io.StreamBuilder;
import org.jabberstudio.jso.io.StreamParseState;

/**
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.4
 */
public class ExtensionBuilder extends AbstractBuilder {
    //Fields
    private Extension       _Ext;
    
    private StreamBuilder   _Extended;
    private List            _ElemStack = new java.util.LinkedList();
    
    //Constructors
    /** Creates a new instance of ExtensionBuilder */
    public ExtensionBuilder(Extension x) throws IllegalArgumentException {
        setExtension(x);
    }

    //Methods
    public Extension getExtension() {
        return _Ext;
    }
    private void setExtension(Extension x) throws IllegalArgumentException {
        if (x == null)
            throw new IllegalArgumentException("Extension cannot be null");
        _Ext = x;
    }

    public NSI getName() {
        return getExtension().getNSI();
    }
    
    protected StreamElement getCurrentElement() {
        return (StreamElement)(_ElemStack.isEmpty() ? null : _ElemStack.get(0));
    }
    protected void pushElement(StreamElement elem) {
        _ElemStack.add(0, elem);
    }
    protected StreamElement popElement() {
        return (StreamElement)(_ElemStack.isEmpty() ? null : _ElemStack.remove(0));
    }

    /** <p>
     * Retrieves the <code>StreamNode</code> this <code>StreamBuilder</code> has
     * been constructing.</p>
     *
     * <p>
     * This method never returns <code>null</code>.</p>
     *
     * @return The new <code>StreamNode</code> that was built.
     * @throws IllegalStateException If the current state is not <code>{@link
     * #COMPLETE}</code>.
     *
     */
    public StreamNode getNode() throws IllegalStateException {
        if (getStatus() != COMPLETE)
            throw new IllegalStateException("Not ready to get the node!");
        return getExtension();
    }
    
    /** <p>
     * Retrieves the <code>StreamBuilder</code> from which to create extended
     * data.</p>
     *
     * <p>
     * This method never returns <Code>null</Code>.</p>
     *
     * @return The <code>StreamBuilder</code> for the extended data.
     * @throws IllegalStateException If the current status is not <code>{@link
     * #EXTENDED_DATA}</code>.
     *
     */
    public StreamBuilder buildExtendedData() throws IllegalStateException {
        return _Extended;
    }
    protected void setExtendedData(StreamBuilder b) {
        _Extended = b;
    }
    
    /** <p>
     * The the given <code>StreamNode</code> as extended data.</p>
     *
     *
     * @param node The extended data to add.
     * @throws IllegalStateException If the current status is not <code>{@link
     * #EXTENDED_DATA}</code>.
     *
     */
    public void addExtendedData(StreamNode node) throws IllegalStateException {
        getExtension().add(node);
    }
    
    /** <p>
     * Resets this <code>StreamBuilder</code>.  By resetting, the status is
     * set to <code>{@link #READY}</code>, and all internal data for building
     * an object is either deleted or set to default values.</p>
     *
     */
    public void reset() {
        Extension       x = getExtension();
        Iterator        itr = x.listNodes().iterator();
        
        setStatus(READY);
        
        while (itr.hasNext()) {
            x.remove((StreamNode)itr.next());
        }
        
        _ElemStack.clear();
    }
    
    /** <p>
     * Processes text data.</p>
     *
     * <p>
     * The value of <code>s</code> is never <code>null</code>.</p>
     *
     * @param s The current <code>StreamParseState</code>.
     * @return The new status for this <code>StreamBuilder</code>.
     *
     */
    public Status processText(StreamParseState s) {
        StreamElement   ctx = s.getContext();
        String          txt = s.getText();
        StreamElement   elem = getCurrentElement();
        Status          status = INVALID;

        try {
            if (elem != null) {
                elem.add(ctx.getDataFactory().createText(txt));

                status = PROCESSING;
            }
        } finally {
            setStatus(status);
        }
        
        return getStatus();
    }
    
    /** <p>
     * Processes "start-tag" data.</p>
     *
     * <p>
     * The value of <code>s</code> is never <code>null</code>.</p>
     *
     * @param s The current <code>StreamParseState</code>.
     * @return The new status for this <Code>StreamBuilder</code>.
     *
     */
    public Status processElementStart(StreamParseState s) {
        StreamElement       ctx = s.getContext();
        NSI                 name = s.getName();
        NSI.Comparator      cmpr = NSI.STRICT_COMPARATOR;
        StreamDataFactory   sdf = ctx.getDataFactory();
        Extension           x = getExtension();
        Set                 attrs = s.getAttributeNames();
        Set                 nss = s.getNamespacePrefixes(s.getDepth());
        Status              status = INVALID;

        try {
            if (NSI.NS_LOOSE_COMPARATOR.compare(name, getName()) == 0) {
                Iterator    attrItr = attrs.iterator();
                Iterator    nssItr = nss.iterator();
                
                //Process all attributes
                while (attrItr.hasNext()) {
                    NSI         attrName = (NSI)attrItr.next();
                    String      val = s.getAttributeValue(attrName);

                    x.setAttributeValue(attrName, val);
                }
                //Process all namespaces
                while (nssItr.hasNext()) {
                    String      nsPre = (String)nssItr.next();
                    String      nsURI = s.getNamespaceURI(nsPre);
                    
                    x.addNamespace(nsPre, nsURI);
                }

                //Push base "element" onto stack
                pushElement(x);
                
                status = PROCESSING;
            } else {
                //Process 
                status = processNestedElementStart(s);
            }
        } finally {
            setStatus(status);
        }
        
        return getStatus();
    }
    
    /** <p>
     * Process "end-tag" data.</p>
     *
     * <p>
     * The value of <code>s</code> is never <code>null</code>.</p>
     *
     * @param s The current <code>StreamParseState</code>.
     * @return The new status for thsi <code>StreamBuilder</code>.
     *
     */
    public Status processElementEnd(StreamParseState s) {
        NSI         name = s.getName();
        Extension   x = getExtension();
        Status      status = INVALID;
        
        try {
            popElement();

            if (NSI.NS_LOOSE_COMPARATOR.compare(name, getName()) == 0) {
                status = COMPLETE;
            } else {
                status = PROCESSING;
            }
        } finally {
            setStatus(status);
        }
        
        return getStatus();
    }

    protected Status processNestedElementStart(StreamParseState s) {
        StreamElement       ctx = s.getContext();
        NSI                 name = s.getName();
        String              uri = name.getNamespaceURI();
        StreamDataFactory   sdf = ctx.getDataFactory();
        StreamBuilder       b;
        Status              status = INVALID;

        b = sdf.createElementBuilder(name, null);
        if (b == null) {
            b = new ElementNode(sdf, name).createBuilder();
        }
        setExtendedData(b);
        status = EXTENDED_DATA;
        
        return status;
    }
}