/*
 * Presence.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso;


import java.util.Locale;

/**
 * <p>
 * Encapsulates a presence element. The precense is used to notify other
 * entities of availability, or to subsribe/unsubsribe from their
 * presence.</p>
 *
 * <p>
 * The presence can have a priority, show message, and/or status message (along
 * with a possible "error" sub-element). The possible types (currently)
 * available to presence are:
 * <ul>
 * <li>{@link #UNAVAILABLE}:   JID is off-line.</li>
 * <li>{@link #INVISIBLE}:     JID is pretending to be off-line
 * (also known as lurking).</li>
 * <li>{@link #PROBE}:         Used to determine current
 * presence (used only by servers).</li>
 * <li>{@link #SUBSCRIBE}:     Request to subscribe to a
 * JID's presence.</li>
 * <li>{@link #SUBSCRIBED}:    Response to a subscription
 * request.</li>
 * <li>{@link #UNSUBSCRIBE}:   Request to unsubscribe to
 * a JID's precense.</li>
 * <li>{@link #UNSUBSCRIBED}:  Response to an
 * unsubscription request.</li>
 * <li>{@link #ERROR}:         The failed result of sending a
 * presence of any other type.</li>
 * </ul></p>
 *
 * @author linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since JSO-0.2
 */
public interface Presence extends Packet {
    //Inner-classes
    /**
     * <p>
     * Enumerates the possible types for <tt>Presence</tt> objects.
     * In general, only the "static final" instances within this class should
     * be used.  "Custom" types can be defined by extending this class.</p>
     *
     * @author  linuxwolf
     * @version 0.13-DEV-20050624155226
     * @since   JSO-0.2
     */
    public static class Type extends Packet.Type {
        /**
         * <p>
         * Constructs a <tt>Type</tt> for the given a type name.</p>
         *
         * <p>
         * The value of <tt>type</tt> cannot be <tt>null</tt> or "", or
         * an <tt>IllegalArgumentException</tt> is thrown.</p>
         *
         * @param type Name of the type.
         * @since   JSO-0.2
         */
        protected Type(String type) {
            super(type);
        }
    }

    //"Constants"
    /** Represents the "unavailable" type for <tt>Presence</tt>.
     * @since   JSO-0.2
     */
    public static final Type    UNAVAILABLE = new Type("unavailable");
    /** Represents the "invisible" type for <tt>Presence</tt>.
     * @since   JSO-0.2
     * @deprecated As of JSO-0.12, this field is deprecated, with no
     * replacement.
     */
    public static final Type    INVISIBLE = new Type("invisible");
    /** Represents the "subscribe" type for <tt>Presence</tt>.
     * @since   JSO-0.2
     */
    public static final Type    SUBSCRIBE = new Type("subscribe");
    /** Represents the "subscribed" type for <tt>Presence</tt>.
     * @since   JSO-0.2
     */
    public static final Type    SUBSCRIBED = new Type("subscribed");
    /** Represents the "unsubscribe" type for <tt>Presence</tt>.
     * @since   JSO-0.2
     */
    public static final Type    UNSUBSCRIBE = new Type("unsubscribe");
    /** Represents the "unsubscribed" type for <tt>Presence</tt>.
     * @since   JSO-0.2
     */
    public static final Type    UNSUBSCRIBED = new Type("unsubscribed");
    /** Represents the "probe" type for <tt>Presence</tt>.
     * @since   JSO-0.2
     */
    public static final Type    PROBE = new Type("probe");
    
    //Methods
    /**
     * <p>
     * Determines if this <tt>Presence</tt> contains a priority.</p>
     *
     * @return <tt>true</tt> if a &lt;priority/&gt; element is present.
     * @since   JSO-0.12
     */
    public boolean hasPriority();
    /**
     * <p>
     * Retieves the priority for this <tt>Presence</tt> element.  The
     * priority is used to help give entities on the same node and domain an
     * ordering.</p>
     *
     * <p>
     * The value returned by this method is in the range -128 to 127. If a
     * priority has not beend set, this method returns 0.</p>
     *
     * @return  The priority for this <tt>Presence</tt>.
     * @since   JSO-0.2
     */
    public int getPriority();
    /**
     * <p>
     * Sets the priority for this <tt>Presence</tt> element.  This method
     * is used to both set and unset the reported priority.</p>
     *
     * <p>
     * The value of <tt>p</tt> must be in the range of -128 to 127, an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param p The new priority for this <tt>Presence</tt>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.2
     */
    public void setPriority(int p);
    /**
     * <p>
     * Removes the priority from this <tt>Presence</tt>. If a priority is no
     * set, this method has not effect.</p>
     *
     * @since   JSO-0.12
     */
    public void removePriority();
    
    /**
     * <p>
     * Retrieves the "show" message for this <tt>Presence</tt>.  This
     * message for describes the entity's availability status.  Typically, it
     * is one of the following values:
     * <ul>
     *   <li>"away":    Entity is temporarily away.</li>
     *   <li>"chat":    Entity is free to chat.</li>
     *   <li>"xa":      Entity is away for an extended period.</li>
     *   <li>"dnd":     Entity is busy.</li>
     * </ul>
     * Values other than the above are generally ignored.</p>
     *
     * <p>
     * the value returned by this method is never <tt>null</tt>, although
     * it may be "".</p>
     *
     * @return The "show" message for this <tt>Presence</tt>.
     * @since   JSO-0.2
     */
    public String getShow();
    /**
     * <p>
     * Sets the "show" message for this <tt>Presence</tt>.  This method is
     * used to both set and unset the "show" message.</p>
     *
     * <p>
     * If the value of <tt>s</tt> is <tt>null</tt> or "", the "show"
     * message is unset.</p>
     *
     * @param s The new "show" message
     * @since   JSO-0.2
     */
    public void setShow(String s);

    /**
     * <p>
     * Determines if this <tt>Presence</tt> includes a status. This method
     * attempts to find the first &lt;status/&gt; element for this
     * <tt>Presence</tt>'s locale (as specified by {@link
     * StreamNode#getLocale}).</p>
     *
     * <p>
     * This method first uses the specific locale (using exactly what (@link
     * StreamNode#getLocale} returns), then uses a language-only variant.</p>
     *
     * @return <tt>true</tt> if an appropriate status is available.
     * @since   JSO-0.12
     */
    public boolean hasStatus();
     /**
     * <p>
     * Determines if this <tt>Presence</tt> includes a status for the given
     * locale.</p>
     *
     * <p>
     * If the value of <tt>l</tt> is <tt>null</tt>, the result from {@link
     * StreamNode#getLocale} for this <tt>Presence</tt> is used.</p>
     *
     * @param l The locale to search by.
     * @return <tt>true</tt> if an appropriate status is available.
     * @since   JSO-0.12
     */
   public boolean hasStatus(Locale l);
    /**
     * <p>
     * Retrieves the status for this <tt>Presence</tt>. This method attempts to
     * find the first &lt;status/&gt; element for this <tt>Presence</tt>'s
     * locale (as specified by {@link StreamNode#getLocale}.</p>
     *
     * <p>
     * This method never returns <tt>null</tt>. If an appropriate status is
     * not available, this method returns "".</p>
     *
     * @return The status of this <tt>Presence</tt>.
     * @since   JSO-0.2
     */    
    public String getStatus();
    /**
     * <p>
     * Retrieves the status for this <tt>Presence</tt> for the given locale.
     * This method attempts to find the first &lt;status/&gt; element for the
     * given locale.</p>
     *
     * <p>
     * This method never returns <tt>null</tt>. If an appropriate status is
     * not available, this method returns "".</p>
     *
     * <p>
     * If the value of <tt>l</tt> is <tt>null</tt>, the result from {@link
     * StreamNode#getLocale} for this <tt>Presence</tt> is used.</p>
     *
     * @param l The locale to search by.
     * @return The status of this <tt>Presence</tt>.
     * @since   JSO-0.12
     */
    public String getStatus(Locale l);
    /**
     * <p>
     * Sets the status for this <tt>Presence</tt>. This method first removes
     * all &lt;status/&gt; elements in the <tt>Presence</tt>'s locale, then
     * adds a &lt;status/&gt; without a declared locale.</p>
     *
     * <p>
     * If the value of <tt>s</tt> is <tt>null</tt>, the "locale default"
     * status is removed.</p>
     *
     * @param s The new status for this <tt>Presence</tt>.
     * @since   JSO-0.2
     */    
    public void setStatus(String s);
    /**
     * <p>
     * Sets the status for this <tt>Presence</tt> for the given locale.</p>
     *
     * <p>
     * If the value of <tt>l</tt> is <tt>null</tt>, the result from {@link
     * StreamNode#getLocale} for this <tt>Presence</tt> is used.</p>
     *
     * <p>
     * If the value of <tt>s</tt> is <tt>null</tt>, the "locale default"
     * status is removed.</p>
     *
     * @param l The locale to search by.
     * @param s The new status for this <tt>Presence</tt>.
     * @since   JSO-0.2
     */    
    public void setStatus(Locale l, String s);
}