/*
 * StreamStatusEvent.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.event;


import org.jabberstudio.jso.Stream;
import org.jabberstudio.jso.StreamContext;

/**
 * <p>
 * Event object for <Code>Stream.Status</code> transitions.  These events are
 * fired "between" the two states, at a specific point in time.  The current
 * status of the <code>Stream</code> returned by {@link #getStream} may not
 * contain either status.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.2
 */
public class StreamStatusEvent extends java.util.EventObject {
    //Fields
    private StreamContext       _Ctx;
    private Stream.Status       _Prev;
    private Stream.Status       _Next;
    private Exception           _Except;
    
    //Constructors
    /**
     * <p>
     * Creates a new <tt>StreamStatusEvent</tt> with the given <tt>Stream</tt>,
     * <tt>StreamContext</tt>, and <tt>Stream.Status</tt> objects.  This
     * constructor assumes that there were no errors causing this
     * transition.</p>
     *
     * <P>
     * The values of <tt>conn</tt>, <tt>ctx</code>, <tt>prev</tt>, and
     * <tt>next</tt> cannot be <tt>null</tt>.</p>
     *
     * @param conn The <tt>Stream</tt> source object.
     * @param ctx The <tt>StreamContext</tt> object.
     * @param prev The previous <code>Stream.Status</code>.
     * @param next The next <code>Stream.Status</code>.
     * @since   JSO-0.10
     */
    public StreamStatusEvent(Stream conn, StreamContext ctx, Stream.Status prev, Stream.Status next) {
        this(conn, ctx, prev, next, null);
    }
    /**
     * <p>
     * Creates a new <tt>StreamStatusEvent</tt> with the given
     * <tt>StreamContext</tt> object, <tt>Stream.Status</tt> objects,
     * and <tt>Exception</tt>.</p>
     *
     * <P>
     * The values of <tt>conn</tt>, <tt>ctx</tt>, <tt>prev</tt>, and
     * <tt>next</tt> cannot be <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>e</tt> may be <tt>null</tt>, which signifies
     * that no errors caused this transition.</p>
     *
     * @param conn The <tt>Stream</tt> source object.
     * @param ctx The <tt>StreamContext</tt> object.
     * @param prev The previous <tt>Stream.Status</tt>.
     * @param next The next <tt>Stream.Status</tt>.
     * @param e The <tt>Exception</tt> for the transition.
     * @since   JSO-0.10
     */
    public StreamStatusEvent(Stream conn, StreamContext ctx, Stream.Status prev, Stream.Status next, Exception e) {
        super(conn);
        _Ctx = ctx;
        _Prev = prev;
        _Next = next;
        _Except = e;
    }
    
    //Methods
    /**
     * <p>
     * Retrieves the source <code>StreamContext</code> for this
     * <Code>StreamStatusEvent</code>.</p>
     *
     * <p>
     * The value returned by this method is never <Code>null</code>.</p>
     *
     * @return The <tt>StreamContext</tt> for this event.
     * @since   JSO-0.2
     */
    public StreamContext getContext() {
        return _Ctx;
    }
    /**
     * <p>
     * Retrieves the source <tt>Stream</tt> for this
     * <tt>StreamStatusEvent</tt>. This is a convenience method over
     * <tt>getSource()</tt> to automatically cast to <tt>Stream</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The source, as a <tt>Stream</tt> type.
     * @since   JSO-0.2
     */
    public Stream getStream() {
        return (Stream)getSource();
    }

    /**
     * <p>
     * Retrieves the previous <Code>Stream.Status</code> for this
     * <code>StreamStatusEvent</code>.  This can also be considered the "from"
     * status.</p>
     *
     * <p>
     * The value returned by this method is never <Code>null</code>.</p>
     *
     * @return The previous <code>Stream.Status</code>.
     * @since   JSO-0.2
     */
    public Stream.Status getPreviousStatus() {
        return _Prev;
    }
    /**
     * <p>
     * Retrieves the next <code>Stream.Status</code> for this
     * <Code>StreamStatusEvent</code>.  This can also be considered the "to"
     * status.</p>
     *
     * <p>
     * The value returned by this method is never <Code>null</code>.</p>
     *
     * @return The next <code>Stream.Status</code>.
     * @since   JSO-0.2
     */
    public Stream.Status getNextStatus() {
        return _Next;
    }
    
    /**
     * <p>
     * Determines if this <code>StreamStatusEvent</code> is the result of
     * exceptional conditions.  This method checks if {@link #getException}
     * will return a non-<code>null</code> value.</p>
     *
     * @return <code>true</code> if {@link #getException} does not return
     * <code>null</code>.
     * @since   JSO-0.2
     */
    public boolean isExceptional() {
        return _Except != null;
    }
    /**
     * <p>
     * Retrieves the <code>Exception</code> for this
     * <code>StreamStatusEvent</code>.</p>
     *
     * <p>
     * The value returned is not <Code>null</code> if {@link #isExceptional}
     * returns <code>true</code>.</p>
     *
     * @return The <code>Exception</code> for this transition.
     * @since   JSO-0.2
     */
    public Exception getException() {
        return _Except;
    }
}