/*
 * SASLMechanismManager.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.sasl;


import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.jabberstudio.jso.util.Service;

/**
 * <p>
 * Class used to create <tt>SASLMechanism</tt> instances.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public class SASLMechanismManager implements SASLMechanismProvider {
    //"Constants"
    /**
     * The name of a property that specifies whether mechanisms that implement
     * forward secrecy between sessions are required. Forward secrecy means
     * that breaking into one session will not automatically provide
     * information for breaking into future sessions. The property contains
     * "true" if mechanisms that implement forward secrecy between sessions
     * are required; "false" if such mechanisms are not required. The default
     * is "false".
     *
     * @since   JSO-0.12
     */
    public static final String  PROPERTY_POLICY_FORWARD_SECRECY     = "javax.security.sasl.policy.forward";
    /**
     * The name of a property that specifies whether mechanisms susceptible to
     * active (non-dictionary) attacks are not permitted. The property contains
     * "true" if mechanisms susceptible to active attacks are not permitted;
     * "false" if such mechanisms are permitted. The default is "false".
     *
     * @since   JSO-0.12
     */
    public static final String  PROPERTY_POLICY_NOACTIVE            = "javax.security.sasl.policy.noactive";
    /**
     * The name of a property that specifies whether mechanisms that accept
     * anonymous login are not permitted. The property contains "true" if
     * mechanisms that accept anonymous login are not permitted; "false" if
     * such mechanisms are permitted. The default is "false".
     *
     * @since   JSO-0.12
     */
    public static final String  PROPERTY_POLICY_NOANONYMOUS         = "javax.security.sasl.policy.noanonymous";
    /**
     * The name of a property that specifies whether mechanisms susceptible to
     * passive dictionary attacks are not permitted. The property contains
     * "true" if mechanisms susceptible to dictionary attacks are not
     * permitted; "false" if such mechanisms are permitted. The default is
     * "false".
     *
     * @since   JSO-0.12
     */
    public static final String  PROPERTY_POLICY_NODICTIONARY        = "javax.security.sasl.policy.nodictionary";
    /**
     * The name of a property that specifies whether mechanisms susceptible to
     * simple plain passive attacks (e.g., "PLAIN") are not permitted. The
     * property contains "true" if such mechanisms are not permitted; "false"
     * if such mechanisms are permitted. The default is "false".
     *
     * @since   JSO-0.12
     */
    public static final String  PROPERTY_POLICY_NOPLAINTEXT         = "javax.security.sasl.policy.noplaintext";
    /**
     * The name of a property that specifies whether mechanisms that pass
     * client credentials are required. The property contains "true" if
     * mechanisms that pass client credentials are required; "false" if such
     * mechanisms are not required. The default is "false".
     *
     * @since   JSO-0.12
     */
    public static final String  PROPERTY_POLICY_PASS_CREDENTIALS    = "javax.security.sasl.policy.credentials";

    /**
     * <p>The name of a property that specifies the quality-of-protection to use.
     * The property contains a comma-separated, ordered list of
     * quality-of-protection values that the client or server is willing to
     * support. The possible values are:</p>
     * <ul>
     * <li><tt>"auth"</tt> - authentication only</li>
     * <li><tt>"auth-int"</tt> - authentication plus integrity protection</li>
     * <li><tt>"auth-conf"</tt> - authentication plus integrity and
     * confidentiality protection</li>
     * </ul>
     *
     * <p>
     * The order of the list specifies the preference order of the client or
     * server. If this property is absent, the default qop is "auth".</p>
     *
     * @since   JSO-0.12
     */
    public static final String  PROPERTY_QOP                        = "javax.security.sasl.qop";
    /**
     * The name of the property that specifies the maximum size of the receive
     * buffer (in bytes). The property contains the string representation of an
     * integer. The default is mechanism-specific.
     *
     * @since   JSO-0.12
     */
    public static final String  PROPERTY_MAX_BUFFER                 = "javax.security.sasl.maxbuffer";
    
    /** The map of default property values.
     *
     * @since   JSO-0.12
     */
    public static final Map     DEFAULT_PROPERTIES;
    
    static {
        Map     temp = new java.util.TreeMap();
        
        temp.put(PROPERTY_POLICY_FORWARD_SECRECY, "false");
        temp.put(PROPERTY_POLICY_NOACTIVE, "false");
        temp.put(PROPERTY_POLICY_NOANONYMOUS, "false");
        temp.put(PROPERTY_POLICY_NODICTIONARY, "false");
        temp.put(PROPERTY_POLICY_NOPLAINTEXT, "false");
        temp.put(PROPERTY_POLICY_PASS_CREDENTIALS, "false");
        
        temp.put(PROPERTY_QOP, "auth");
        
        DEFAULT_PROPERTIES = java.util.Collections.unmodifiableMap(temp);
    }
    
    //Fields
    private Service      _SASLService = new Service(SASLMechanismProvider.class, null);
    private List         _Providers = new java.util.LinkedList();
    
    private static SASLMechanismManager _Inst;
    
    //Constructors
    /**
     * <p>
     * Creates a new <tt>SASLMechanismManager</tt>.  When created, it loads all
     * <tt>SASLMechanismProvider</tt> instances via SPI.</p>
     *
     * @since   JSO-0.12
     */
    protected SASLMechanismManager() {
        _Providers = loadProviders();
    }
    
    //Methods
    /**
     * <p>
     * Retrieves the static instance of <tt>SASLMechanismManager</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The <tt>SASLMechanismManager</tt> singleton.
     * @since   JSO-0.12
     */
    public static synchronized SASLMechanismManager getInstance() {
        if (_Inst == null)
            _Inst = new SASLMechanismManager();
        return _Inst;
    }

    /**
     * <p>
     * Registers the given <tt>SASLMechanismProvider</tt> to this
     * <tt>SASLMechanismManager</tt>.</p>
     *
     * <p>
     * The value of <tt>provider</tt> may be <tt>null</tt>, in which case no
     * actions are performed. This method does not attempt to verify if
     * <tt>provider</tt> is already registered.</p>
     *
     * @param provider The <tt>SASLMechanismProvider</tt> to register.
     * @since   JSO-0.12
     */
    public synchronized void registerProvider(SASLMechanismProvider provider) {
        if (provider != null)
            _Providers.add(0, provider);
    }
    /**
     * <p>
     * Unregisters the given <tt>SASLMechanismProvider</tt> from this
     * <tt>SASLMechanismManager</tt>.</p>
     *
     * <p>
     * The value of <tt>provider</tt> may be <tt>null</tt>, in which case no
     * actions are performed. This method does not attempt to verify if
     * <tt>provider</tt> is registered more than once; only the first occurance
     * is removed.</p>
     *
     * @param provider The <tt>SASLMechanismProvider</tt> to unregister.
     * @since   JSO-0.12
     */
    public synchronized void unregisterProvider(SASLMechanismProvider provider) {
        if (provider != null)
            _Providers.add(provider);
    }
    
    /**
     * <p>
     * Retrieves the list of client mechanism names, within the constraints of
     * the given properties. This method iterates over the list of
     * <tt>SASLMechanismProvider</tt>s, and calls {@link
     * SASLMechanismProvider#getClientMechanismNames} to generate the list.
     * It ensures that no mechanism name is listed more than once.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. Each element
     * in the returned list is a string.</p>
     *
     * <p>
     * The value of <tt>props</tt> may be <tt>null</tt>, in which case an
     * empty map is used.</p>
     *
     * @param props The map of authentication properties.
     * @return The list of supported client mechanisms.
     * @since   JSO-0.12
     */
    public synchronized List getClientMechanismNames(Map props) {
        Iterator                itr = _Providers.iterator(), mechItr;
        SASLMechanismProvider   provider;
        String                  name;
        List                    mechs = new java.util.ArrayList();
        
        //Validate Input
        if (props == null)
            props = Collections.EMPTY_MAP;

        //Iterate and append
        while (itr.hasNext()) {
            provider = (SASLMechanismProvider)itr.next();
            mechItr = provider.getClientMechanismNames(props).iterator();
            
            while (mechItr.hasNext()) {
                name = (String)mechItr.next();
                
                if (!mechs.contains(name))
                    mechs.add(name);
            }
        }
        
        return mechs;
    }
    /**
     * <p>
     * Creates a new client <tt>SASLMechanism</tt> using the given
     * parameters. This method creates a client <tt>SASLMechanism</tt> for the
     * first supported mechanism and within the constraints of the given
     * properties. It iterates of the list of registered
     * <tt>SASLMechanismProvider</tt>s, and calls {@link
     * SASLMechanismProvider#createClientMechanism} until a non-<tt>null</tt>
     * instance is returned.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if a client
     * <tt>SASLMechanism</tt> cannot be created.</p>
     *
     * <p>
     * The value of <tt>ci</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown. Also, if the call to {@link
     * SASLClientInfo#isValid} on <tt>ci</tt> returns <tt>false</tt>, an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param ci The <tt>SASLClientInfo</tt> to create by.
     * @return The client <tt>SASLMechanism</tt> appropriate for the given
     * <tt>SASLClientInfo</tt>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public synchronized SASLMechanism createClientMechanism(SASLClientInfo ci) throws IllegalArgumentException {
        Iterator                itr = _Providers.iterator();
        SASLMechanismProvider   provider;
        SASLMechanism           impl = null;
        
        //Validate input
        if ((ci == null) || !ci.isValid())
            throw new IllegalArgumentException("SASLClientInfo must be non-null and valid");
        
        while ((impl == null) && itr.hasNext()) {
            provider = (SASLMechanismProvider)itr.next();
            
            impl = provider.createClientMechanism(ci);
        }
        
        return impl;
    }
    
    /**
     * <p>
     * Retrieves the list of server mechanism names, within the constraints of
     * the given properties. This method iterates over the list of
     * <tt>SASLMechanismProvider</tt>s, and calls {@link
     * SASLMechanismProvider#getServerMechanismNames} to generate the list.
     * It ensures that no mechanism name is listed more than once.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. Each element
     * in the returned list is a string.</p>
     *
     * <p>
     * The value of <tt>props</tt> may be <tt>null</tt>, in which case an
     * empty map is used.</p>
     *
     * @param props The map of authentication properties.
     * @return The list of supported server mechanisms.
     * @since   JSO-0.12
     */
    public synchronized List getServerMechanismNames(Map props) {
        Iterator                itr = _Providers.iterator();
        SASLMechanismProvider   provider;
        List                    mechs = new java.util.ArrayList();
        
        //Validate Input
        if (props == null)
            props = Collections.EMPTY_MAP;

        //Iterate and append
        while (itr.hasNext()) {
            provider = (SASLMechanismProvider)itr.next();
            
            mechs.addAll(provider.getClientMechanismNames(props));
        }
        
        return mechs;
    }
    /**
     * <p>
     * Creates a new server <tt>SASLMechanism</tt> using the given
     * parameters. This method server a client <tt>SASLMechanism</tt> for the
     * first supported mechanism and within the constraints of the given
     * properties. It iterates over the list of registered
     * <tt>SASLMechanismProvider</tt>s and calls {@link
     * SASLMechanismProvider#createServerMechanism} until a non-<tt>null</tt>
     * instance is returned.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if a server
     * <tt>SASLMechanism</tt> cannot be created.</p>
     *
     * <p>
     * The value of <tt>si</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown. Also, if the call to {@link
     * SASLClientInfo#isValid} on <tt>si</tt> returns <tt>false</tt>, an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param si The <tt>SASLServerInfo</tt> to create by.
     * @return The server <tt>SASLMechanism</tt> appropriate for the given
     * <tt>ServerSupport</tt>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public synchronized SASLMechanism createServerMechanism(SASLServerInfo si) throws IllegalArgumentException {
        Iterator                itr = _Providers.iterator();
        SASLMechanismProvider   provider;
        SASLMechanism           impl = null;
        
        //Validate input
        if ((si == null) || !si.isValid())
            throw new IllegalArgumentException("SASLServerInfo must be non-null and valid");
        
        while ((impl == null) && itr.hasNext()) {
            provider = (SASLMechanismProvider)itr.next();
            
            impl = provider.createServerMechanism(si);
        }
        
        return impl;
    }
    
    private List loadProviders() {
        Map                     vals = _SASLService.getProviders();
        Iterator                itr = vals.keySet().iterator();
        String                  cls;
        SASLMechanismProvider   provider;
        List                    provs = new java.util.ArrayList(vals.size());

        while (itr.hasNext()) {
            cls = (String)itr.next();
            provider = (SASLMechanismProvider)_SASLService.obtainProvider(cls);

            provs.add(0, provider);
        }
        
        return provs;
    }
}