/*
 * IdentityGenerator.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.util;


/**
 * <p>
 * A factory for generating sequentially unigue identifiers. This is iodeal
 * for use with the "id" attribute of <tt>StreamElement</tt>.</p>
 *
 * <p>
 * As of JSO-0.10, this class no longer generates randomly unique identifer,
 * instead using an internal sequence. It also generates a "smaller" size
 * identifier, which should be more compatible with existing software.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.2
 */
public class IdentityGenerator {
    //"Constants"
    /** Default hash value for {@link #generate(String)}.
     *
     * @since   JSO-0.10
     */
    public static final String DEFAULT_HASH = "JSO-" + Utilities.VERSION;
    
    //Fields
    private transient long      _Current;
    private StringBuffer        _Buffer = new StringBuffer();
    
    private static IdentityGenerator    _Instance;

    //Constructors
    /**
     * <p>
     * Creates a new <tt>IdentityGenerator</tt>.</p>
     *
     * @since   JSO-0.2
     */
    public IdentityGenerator() {
        _Current = 1;
    }

    //Methods
    /**
     * <p>
     * Generates the next global identifier. This method behaves exactly as if
     * calling {@link #generateGlobal(String)} with a <tt>null</tt> value for
     * <tt>base</tt>.</p>
     *
     * <p>
     * <b>NOTE:</t> the identifier generated by this method is not guaranteed
     * to be absolutely unique for all other instances of this class. It is
     * guaranteed to be unique for calls to this method or to {@link
     * #generateGlobal(String)}.</p>
     *
     * @return The next string identifier.
     * @since   JSO-0.12
     */
    public static String generateGlobal() {
        return generateGlobal(null);
    }
    /**
     * <p>
     * Generates the next global identifier. The generated identifer
     * incorporates the given base string and a sequential number.</p>
     *
     * <p>
     * <b>NOTE:</t> the identifier generated by this method is not guaranteed
     * to be absolutely unique for all other instances of this class. It is
     * guaranteed to be unique for calls to this method or to {@link
     * #generateGlobal()}.</p>
     *
     * <p>
     * The value of <tt>base</tt> may be <tt>null</tt> or "", in which case the
     * value of {@link #DEFAULT_HASH} is used.</p>
     *
     * @param base The base string to incorporate.
     * @return The next string identifier.
     * @since   JSO-0.12
     */
    public static String generateGlobal(String base) {
        if (_Instance == null)
            _Instance = new IdentityGenerator();
        
        return _Instance.generate(base);
    }
    
    /**
     * <p>
     * Generates the next identifier from this <tt>IdentityGenerator</tt>.
     * This method behaves exactly as if calling {@link #generate(String)} with
     * a <tt>null</tt> value for <tt>base</tt>.</p>
     *
     * @return The next string identifier.
     * @since   JSO-0.10
     */
    public synchronized String generate() {
        return generate(null);
    }
    /**
     * <p>
     * Generates the next identifier from this <tt>IdentityGenerator</tt>.
     * The generated identifer incorporates the given base string and a
     * sequential number.</p>
     *
     * <p>
     * The value of <tt>base</tt> can be <tt>null</tt> or "".  If so,
     * the value of {@link #DEFAULT_HASH} is used.</p>
     *
     * @param hash The base string to incorporate.
     * @return The next string identifier.
     * @since   JSO-0.2
     */
    public synchronized String generate(String hash) {
        StringBuffer    buffer = _Buffer;

        if (hash == null)
            hash = DEFAULT_HASH;

        buffer.setLength(0);
        buffer.append(hash).append("-").append(_Current++);

        return buffer.toString();
    }
    
    public String toString() {
        return generate();
    }
}