/*
 * PacketListenerRelay.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.util;


import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import org.jabberstudio.jso.event.PacketDispatcher;
import org.jabberstudio.jso.event.PacketEvent;
import org.jabberstudio.jso.event.PacketListener;

/**
 * <p>
 * Class to relay <tt>PacketEvent</tt>s to multiple
 * <tt>PacketListener</tt>s.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.8
 */
public class PacketListenerRelay implements PacketDispatcher, PacketListener {
    //Fields
    private List            _RcvRelay = new java.util.LinkedList();
    private List            _SndRelay = new java.util.LinkedList();
    
    //Constructors
    /**
     * <p>
     * Creates a new <tt>PacketListenerRelay</tt>.</p>
     *
     * @since   JSO-0.8
     */
    public PacketListenerRelay() {
    }
    
    //Methods
    /**
     * <p>
     * Retrieves the complete list of <tt>PacketListener</tt> objects
     * registered to this <tt>PacketListenerRelay</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. Each element
     * in the returned list is a non-<tt>null</tt> instance of
     * <tt>PacketListener</tt>.</p>
     *
     * @return The list of registered <tt>PacketListener</tt> objects.
     * @since   JSO-0.12
     */
    public List getPacketListeners() {
        return getPacketListeners(null);
    }
    /**
     * <p>
     * Retrieves the list of <tt>PacketListener</tt> objects registered to this
     * <tt>PacketListenerRelay</tt> for the given type.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. Each element
     * in the returned list is a non-<tt>null</tt> instance of
     * <tt>PacketListener</tt>.</p>
     *
     * <p>
     * The value of <tt>type</tt> may be <tt>null</tt>, in which case all
     * <tt>PacketListener</tt> instances registered (regardless of direction)
     * are returned.</p>
     *
     * @param type The <tt>PacketEvent.Type</tt> to retrieve for.
     * @return The list of registered <tt>PacketListener</tt> objects.
     * @since   JSO-0.12
     */
    public synchronized List getPacketListeners(PacketEvent.Type type) {
        List    listeners = new java.util.ArrayList();
        
        if          ((type == null) || (type == PacketEvent.RECEIVED))
            listeners.addAll(_RcvRelay);
        else if     ((type == null) || (type == PacketEvent.SENT))
            listeners.addAll(_SndRelay);
        
        return listeners;
    }
    
    /**
     * <p>
     * Registers the given <tt>PacketListener</tt> for any packet
     * transfer events.  The given listener is registered to events for both
     * received and sent <tt>Packet</tt> objects.</p>
     *
     * <p>
     * The value of <tt>l</tt> should not be <tt>null</tt>.  No actions
     * are performed if this is true.  If <tt>l</tt> is already registered,
     * it may again be registered.</p>
     *
     * @param l The <tt>PacketListener</tt> to register.
     * @since   JSO-0.8
     *
     */
    public void addPacketListener(PacketListener l) {
        addPacketListener(null, l);
    }
    /**
     * <p>
     * Registers the given <tt>PacketListener</tt> for packet transfer
     * events of the given type.  The given listener is registered only to
     * events matching <tt>t</tt>.</p>
     *
     * <p>
     * The value of <tt>t</tt> can be <tt>null</tt>, in which case this
     * method performs identically to {@link
     * #addPacketListener(PacketListener)}.</p>
     *
     * <p>
     * The value of <tt>l</tt> should not be <tt>null</tt>.  No actions
     * are performed if this is true.  If <tt>l</tt> is already registered,
     * it is registered again.</p>
     *
     * @param type The <tt>PacketEvent.Type</tt> to register for.
     * @param l The <tt>PacketListener</tt> to register.
     * @since   JSO-0.8
     *
     */
    public synchronized void addPacketListener(PacketEvent.Type type, PacketListener l) {
        if (l != null) {
            if          ((type == null) || (type == PacketEvent.RECEIVED))
                _RcvRelay.add(l);
            if          ((type == null) || (type == PacketEvent.SENT))
                _SndRelay.add(l);
        }
    }
    /**
     * <p>
     * Unregisters the given <tt>PacketListener</tt> for any packet
     * transfer events.  The given listener is unregistered from events for both
     * received and sent <tt>Packet</tt> objects.</p>
     *
     * <p>
     * The value of <tt>l</tt> should not be <tt>null</tt>.  No actions
     * are performed if this is true.  If <tt>l</tt> is registered more
     * than once, only one is unregistered.</p>
     *
     * @param l The <tt>PacketListener</tt> to unregister.
     * @since   JSO-0.8
     *
     */
    public void removePacketListener(PacketListener l) {
        removePacketListener(null, l);
    }
    /**
     * <p>
     * Unegisters the given <tt>PacketListener</tt> for packet transfer
     * events of the given type.  The given listener is unregistered only from
     * events matching <tt>t</tt>.</p>
     *
     * <p>
     * The value of <tt>t</tt> can be <tt>null</tt>, in which case this
     * method performs identically to {@link
     * #removePacketListener(PacketListener)}.</p>
     *
     * <p>
     * The value of <tt>l</tt> should not be <tt>null</tt>.  No actions
     * are performed if this is true.  If <tt>l</tt> is registered more
     * than once, only one is unregistered.</p>
     *
     * @param type The <tt>PacketEvent.Type</tt> to register for.
     * @param l The <tt>PacketListener</tt> to register.
     * @since   JSO-0.8
     */
    public synchronized void removePacketListener(PacketEvent.Type type, PacketListener l) {
        if (l != null) {
            if      ((type == null) || (type == PacketEvent.RECEIVED))
                _RcvRelay.remove(l);
            if      ((type == null) || (type == PacketEvent.SENT))
                _SndRelay.remove(l);
        }
    }

    /** <p>
     * Method called when a <tt>Packet</tt> is received or sent. This 
     * implementation iterates over the list of registered
     * <tt>PacketListener</tt>s, calling {@link
     * PacketListener#packetTransferred} on each using <tt>evt</tt>.</p>
     *
     * @param evt The <tt>PacketEvent</tt> object.
     * @since   JSO-0.8
     */
    public void packetTransferred(PacketEvent evt) {
        PacketEvent.Type    type = evt.getType();
        Iterator            itr = getPacketListeners(type).iterator();
        Logger              log = obtainLogger();

        if (log.isDebugEnabled())
            log.debug("relaying packet transferred for " + this);

        while (itr.hasNext()) {
            ((PacketListener)itr.next()).packetTransferred(evt);
        }
    }
    
    /**
     * <p>
     * Obtains a Logger for this <tt>PacketListenerRelay</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The logger for this <tt>PacketListenerRelay</tt>.</p>
     * @since   JSO-0.12
     */
    protected Logger obtainLogger() {
        return Logger.getLogger(getClass());
    }
}