/*
 * XPathListener.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.util;

import java.util.Map;
import java.util.Set;

import org.jaxen.XPath;

import org.saxpath.SAXPathException;

import org.jabberstudio.jso.Packet;
import org.jabberstudio.jso.StreamContext;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.event.PacketEvent;
import org.jabberstudio.jso.event.PacketListener;
import org.jabberstudio.jso.xpath.XPathSupport;

/**
 * <p>
 * A <tt>PacketListener</tt> proxy to process events based on an XPath
 * expression context. Every <tt>PacketEvent</tt> passed to this class to
 * process is evaluated against the contained expression. If a match occurs,
 * the proxied <tt>PacketListener</tt> is called.</p>
 *
 * <p>
 * This class can be created wither with an <tt>XPath</tt> object or an
 * XPath expression and a <tt>XPathSupport</tt>. In the case of the
 * latter, the expression is modified to be relative to the transferred
 * <tt>Packet</tt>'s parent. In both cases, a copy of the
 * <tt>Packet</tt> to evaluate is placed in a "sandbox" container.</p>
 *
 * <p>
 * Instances of this class can be registered with more than one
 * <tt>PacketDispatcher</tt>. There are no threading issues involved.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.8
 */
public class XPathListener implements PacketListener {
    //Fields
    private PacketListener  _Match;
    private XPath           _XPath;
    
    //Constructors
    /**
     * <p>
     * Creates a new <tt>XPathListener</tt> with a relative XPath
     * expression. <tt>support</tt> and <tt>expr</tt> are used to create
     * the <tt>XPath</tt> object. The expression is modified (if necessary)
     * to be relative to the transferred <tt>Packet</tt>'s (internal)
     * parent.</p>
     *
     * <p>
     * The value of <tt>support</tt> cannot be <tt>null</tt>,
     * or an <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>expr</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown. If <tt>expr</tt>,
     * after formatting, does not evaluate to a valid XPath expression, a
     * <tt>SAXPathException</tt> is thrown.</p>
     *
     * @param support The <tt>XPathSupport</tt> to use for XPath creation.
     * @param expr The relative XPath expression to process.
     * @throws IllegalArgumentException If any of the parameters are invalid.
     * @throws SAXPathException If the expression does not evaluate to a valid
     * XPath.
     * @since   JSO-0.8.4
     */
    public XPathListener(XPathSupport support, String expr) throws IllegalArgumentException, SAXPathException {
        this(support, expr, null, null);
    }
    /**
     * <p>
     * Creates a new <tt>XPathListener</tt> with a relative XPath
     * expression. <tt>support</tt> and <tt>expr</tt> are used to create
     * the <tt>XPath</tt> object, and <tt>nss</tt> is used to set up namespace
     * mappings. The expression is modified (if necessary) to be relative to
     * the transferred <tt>Packet</tt>'s (internal) parent.</p>
     *
     * <p>
     * The value of <tt>support</tt> cannot be <tt>null</tt>,
     * or an <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>expr</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown. If <tt>expr</tt>,
     * after formatting, does not evaluate to a valid XPath expression, a
     * <tt>SAXPathException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>nss</tt> may be <tt>null</tt>, in which case it is
     * considered to be an empty Map. For each entry in the given Map, the
     * key is considered the namespace prefix, and the value is the namespace
     * URI.</p>
     *
     * @param support The <tt>XPathSupport</tt> to use for XPath creation.
     * @param expr The relative XPath expression to process.
     * @param nss The namespace mapping to apply.
     * @throws IllegalArgumentException If any of the parameters are invalid.
     * @throws SAXPathException If the expression does not evaluate to a valid
     * XPath.
     * @since   JSO-0.12
     */
    public XPathListener(XPathSupport support, String expr, Map nss) throws IllegalArgumentException, SAXPathException {
        this(support, expr, nss, null);
    }
    /**
     * <p>
     * Creates a new <tt>XPathListener</tt> with a relative XPath
     * expression. <tt>support</tt> and <tt>expr</tt> are used to create
     * the <tt>XPath</tt> object. The expression is modified (if necessary)
     * to be relative to the transferred <tt>Packet</tt>'s (internal)
     * parent.</p>
     *
     * <p>
     * The value of <tt>support</tt> cannot be <tt>null</tt>,
     * or an <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>expr</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown. If <tt>expr</tt>,
     * after formatting, does not evaluate to a valid XPath expression, a
     * <tt>SAXPathException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>match</tt> may be <tt>null</tt>.</p>
     *
     * @param support The <tt>XPathSupport</tt> to use for XPath creation.
     * @param expr The relative XPath expression to process.
     * @param match The <tt>PacketListener</tt> to use if a match is found.
     * @throws IllegalArgumentException If any of the parameters are invalid.
     * @throws SAXPathException If the expression does not evaluate to a valid
     * XPath.
     * @since   JSO-0.8
     */
    public XPathListener(XPathSupport support, String expr, PacketListener match) throws IllegalArgumentException, SAXPathException {
        this(createXPath(support, expr), match);
    }
    /**
     * <p>
     * Creates a new <tt>XPathListener</tt> with a relative XPath
     * expression. <tt>support</tt> and <tt>expr</tt> are used to create
     * the <tt>XPath</tt> object, and <tt>nss</tt> is used to set up namespace
     * mappings. The expression is modified (if necessary) to be relative to
     * the transferred <tt>Packet</tt>'s (internal) parent.</p>
     *
     * <p>
     * The value of <tt>support</tt> cannot be <tt>null</tt>,
     * or an <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>expr</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown. If <tt>expr</tt>,
     * after formatting, does not evaluate to a valid XPath expression, a
     * <tt>SAXPathException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>nss</tt> may be <tt>null</tt>, in which case it is
     * considered to be an empty Map. For each entry in the given Map, the
     * key is considered the namespace prefix, and the value is the namespace
     * URI.</p>
     *
     * <p>
     * The value of <tt>match</tt> may be <tt>null</tt>.</p>
     *
     * @param support The <tt>XPathSupport</tt> to use for XPath creation.
     * @param expr The relative XPath expression to process.
     * @param nss The namespace mapping to apply.
     * @param match The <tt>PacketListener</tt> to use if a match is found.
     * @throws IllegalArgumentException If any of the parameters are invalid.
     * @throws SAXPathException If the expression does not evaluate to a valid
     * XPath.
     * @since   JSO-0.12
     */
    public XPathListener(XPathSupport support, String expr, Map nss, PacketListener match) throws IllegalArgumentException, SAXPathException {
        this(createXPath(support, expr), match);
        
        setupNamespaces(nss);
    }
    /**
     * <p>
     * Creates a new <tt>XPathListener</tt> with the given
     * <tt>XPath</tt> object.</p>
     *
     * <p>
     * The value of <tt>path</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param path The <tt>XPath</tt> to use for evaluations.
     * @throws IllegalArgumentException If any of the parameters are invalid.
     * @since   JSO-0.8.4
     */
    public XPathListener(XPath path) throws IllegalArgumentException {
        this(path, null);
    }
    /**
     * <p>
     * Creates a new <tt>XPathListener</tt> with the given
     * <tt>XPath</tt> object.</p>
     *
     * <p>
     * The value of <tt>path</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>match</tt> may be <tt>null</tt>.</p>
     *
     * @param path The <tt>XPath</tt> to use for evaluations.
     * @param match The <tt>PacketListener</tt> to use if a match is found.
     * @throws IllegalArgumentException If any of the parameters are invalid.
     * @since   JSO-0.8
     */
    public XPathListener(XPath path, PacketListener match) throws IllegalArgumentException {
        setXPath(path);
        setMatchedListener(match);
    }
    
    //Methods
    /**
     * <p>
     * Retrieves the <tt>XPath</tt> object to evaluate against. This method
     * allows setup of the various XPath contexts (namespace, function, and
     * variable).</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The <tt>XPath</tt> used for evaluation.
     * @since   JSO-0.8
     */
    public XPath getXPath() {
        return _XPath;
    }
    private void setXPath(XPath path) throws IllegalArgumentException {
        if (path == null)
            throw new IllegalArgumentException("XPath object cannot be null");
        _XPath = path;
    }
    
    /**
     * <p>
     * Retrieves the <tt>PacketListener</tt> used for "matched"
     * <tt>PacketEvent</tt> instances.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt>.</p>
     *
     * @return The <tt>PacketListener</tt> used for processing "matches".
     * @since   JSO-0.8
     */
    public PacketListener getMatchedListener() {
        return _Match;
    }
    private void setMatchedListener(PacketListener match) {
        _Match = match;
    }
    
    /**
     * <p>
     * Adds the set of namespaces to the XPath. The namespaces are
     * determined from <tt>ns</tt>, which is a set of {@link
     * org.jabberstudio.jso.StreamNamespace} objects.</p>
     *
     * <p>
     * The value of <tt>nss</tt> may be <tt>null</tt>, in which case it is
     * considered to be an empty Set.</p>
     *
     * @param nss The namespace set.
     * @throws RuntimeException If any errors occur adding namespaces to the
     * XPath.
     * @since   JSO-0.8
     */
    public void setupNamespaces(Set nss) throws RuntimeException {
        Utilities.setupNamespaces(getXPath(), nss);
    }
    /**
     * <p>
     * Adds the map of namespaces to the XPath. The namespaces are
     * determined from <tt>nss</tt>, using the key as the namespace
     * prefix, and the value as the namespace URI.</p>
     *
     * <p>
     * The value of <tt>nss</tt> may be <tt>null</tt>, in which case it is
     * considered to be an empty Map.</p>
     *
     * @param nss The namespace map.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @throws RuntimeException If any errors occur adding namespaces to the
     * XPath.
     * @since   JSO-0.8
     */
    public void setupNamespaces(Map nss) throws RuntimeException {
        Utilities.setupNamespaces(getXPath(), nss);
    }

    /**
     * <p>
     * Called when a received or sent <tt>Packet</tt> matches the given
     * XPath expression.</p>
     *
     * <p>
     * This method attempts to pass processing onto the contained
     * <tt>PacketListener</tt>. If {@link #getMatchedListener} returns
     * <tt>null</tt>, this method does nothing.</p>
     *
     * @param evt The <tt>PacketEvent</tt> that matches.
     */
    public void packetMatched(PacketEvent evt) {
        PacketListener  l = getMatchedListener();
        
        if (l != null)
            l.packetTransferred(evt);
    }
    /**
     * <p>
     * Called when a <tt>Packet</tt> node is sent and/or received.</p>
     *
     * <p>
     * This method evaluates the {@link PacketEvent#getData} against the
     * <tt>XPath</tt>. If the evaluation returns a non-<tt>null</tt> value, it
     * passes processing onto {@link #packetMatched}.</p>
     *
     * @param evt The <tt>PacketEvent</tt> object.
     * @since   JSO-0.8
     */
    public void packetTransferred(PacketEvent evt) {
        StreamContext       ctx = evt.getContext();
        StreamDataFactory   sdf = ctx.getDataFactory();
        StreamElement       base = sdf.createElementNode(sdf.createNSI("stream", ctx.getNamespaceURI("")));
        StreamElement       target = (Packet)evt.getData();
        StreamElement       parent = target.getParent();
        
        target.setParent(null);
        base.add(target);
        try {
            if (getXPath().selectSingleNode(base) != null) {
                base.remove(target);
                target.setParent(parent);
                packetMatched(evt);
            }
        } catch (Throwable t) {
        }
        target.setParent(parent);
    }
    
    private static final XPath createXPath(XPathSupport support, String expr) throws IllegalArgumentException, SAXPathException {
        //Validate input
        if (support == null)
            throw new IllegalArgumentException("XPathSupport cannot be null");
        if (!Utilities.isValidString(expr))
            throw new IllegalArgumentException("XPath expression cannot be null or \"\"");

        //Configure expression
        if (!expr.startsWith("./"))
            expr = (expr.startsWith("/") ? "." : "./") + expr;
        
        return support.createXPath(expr);
    }
}