/*
 * RegisterQuery.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.x.core;


import java.net.URL;
import java.util.Set;

import org.jabberstudio.jso.Extension;
import org.jabberstudio.jso.NSI;

/**
 * <p>
 * Interface for representing "jabber:iq:register" information.</p>
 *
 * <p>
 * Note that any modifications to the data contained by an instance of
 * <code>RegisterQuery</code> is not automatically reflected back to the
 * "server" or "component".  The <code>RegisterQuery</code> (wrapped in an
 * appropriate <code>InfoQuery</code>) must still be sent for changes to have
 * any real effect.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.4
 */
public interface RegisterQuery extends Extension {
    //"Constants"
    /**
     * Namespace URI for <tt>RegisterQuery</tt> (jabber:iq:register).
     * @since   JSO-0.4
     */
    public static final String  NAMESPACE = "jabber:iq:register";
    /**
     * Qualified name for <tt>RegisterQuery</tt> ({jabber:iq:register}query).
     * @since   JSO-0.12
     */
    public static final NSI     NAME = new NSI("query", NAMESPACE);
    
    //Methods
    /**
     * <p>
     * Retrieves the instructions for this <code>RegisterQuery</code>.</p>
     *
     * <p>
     * The value returned by this method may be <code>null</code>, if no
     * instructions are currently available.</p>
     *
     * @return The instructions for this <code>RegisterQuery</code>.
     * @since   JSO-0.4
     */
    public String getInstructions();
    /**
     * <p>
     * Sets the instructions for this <code>RegisterQuery</code>.</p>
     *
     * <p>
     * The value of <code>inst</code> may be <code>null</code>, in which case
     * the &lt;instructions/&gt; element is removed.</p>
     *
     * @param inst The new instructions.
     * @since   JSO-0.4
     */
    public void setInstructions(String inst);
    
    /**
     * <p>
     * Determines if this <code>RegisterQuery</code> is already "registered".
     * If this method returns <code>true</code>, the information contained
     * in this <code>RegisterQuery</code> updates an existing account.</p>
     *
     * @return <code>true</code> if this represents a "registered" account.
     * @since   JSO-0.4
     */
    public boolean isRegistered();
    /**
     * <p>
     * Sets the "registered" state for this <Code>RegisterQuery</code>.</p>
     *
     * <p>
     * If the value of <code>reg</code> is <code>true</code> a
     * &lt;registered/&gt; element is added to the XML, otherwise an existing
     * one (if any) is removed.</p>
     *
     * @param reg The new "registered" state.
     * @since   JSO-0.4
     */
    public void setRegistered(boolean reg);
    
    /**
     * <p>
     * Determines if this <code>RegisterQuery</code> flags removing an
     * account.  If this method returns <code>true</code>, the information
     * contained by this <code>RegisterQuery</code> is (or has been)
     * removed.</p>
     *
     * @return <code>true</code> if this represents a "removed" account.
     * @since   JSO-0.4
     */
    public boolean isRemove();
    /**
     * <p>
     * Sets the "removed" state for this <code>RegisterQuery</code>.</p>
     *
     * <p>
     * If the value of <code>rem</code> is <code>true</code>, a
     * &lt;remove/&gt; element is added to the XML, otherwise an existing one
     * (if any) is removed.</p>
     *
     * @param rem The new "removed" state.
     * @since   JSO-0.4
     */
    public void setRemove(boolean rem);
    
    /**
     * <p>
     * Retrieves the key for this <tt>RegisterQuery</tt>. This corresponds to
     * the field "key".</p>
     *
     * <p>
     * This method returns <tt>null</tt> if the key has not been set.</p>
     *
     * @return The key for this <tt>RegisterQuery</tt>.
     * @since   JSO-0.12
     */
    public String getKey();
    /**
     * <p>
     * Sets the key for this <tt>RegisterQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>key</tt> may be <tt>null</tt> or "". Note that calling
     * this method always results in the inclusion of the "key" field.
     * Call {@link #clearKey} to remove this field.</p>
     *
     * @param key The new key.
     * @since   JSO-0.12
     */
    public void setKey(String key);
    /**
     * <p>
     * Removes the key from this <tt>RegisterQuery</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public void clearKey();
    
    /**
     * <p>
     * Retrieves the username for this <tt>RegisterQuery</tt>. This corresponds
     * to the field "username".</p>
     *
     * <p>
     * This method returns <tt>null</tt> if the username has not been set.</p>
     *
     * @return The username for this <tt>RegisterQuery</tt>.
     * @since   JSO-0.12
     */
    public String getUsername();
    /**
     * <p>
     * Sets the username for this <tt>RegisterQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>usr</tt> may be <tt>null</tt> or "". Note that calling
     * this method always results in the inclusion of the "username" field.
     * Call {@link #clearUsername} to remove this field.</p>
     *
     * @param usr The new username.
     * @since   JSO-0.12
     */
    public void setUsername(String usr);
    /**
     * <p>
     * Removes the username from this <tt>RegisterQuery</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public void clearUsername();
    
    /**
     * <p>
     * Retrieves the password for this <tt>RegisterQuery</tt>. This corresponds
     * to the field "password".</p>
     *
     * <p>
     * This method returns <tt>null</tt> if the password has not been set.</p>
     *
     * @return The password for this <tt>RegisterQuery</tt>.
     * @since   JSO-0.12
     */
    public String getPassword();
    /**
     * <p>
     * Sets the password for this <tt>RegisterQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>pass</tt> may be <tt>null</tt> or "". Note that calling
     * this method always results in the inclusion of the "password" field.
     * Call {@link #clearPassword} to remove this field.</p>
     *
     * @param pass The new password.
     * @since   JSO-0.12
     */
    public void setPassword(String pass);
    /**
     * <p>
     * Removes the password from this <tt>RegisterQuery</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public void clearPassword();
    
    /**
     * <p>
     * Retrieves the nickname for this <tt>RegisterQuery</tt>. This corresponds
     * to the field "nick".</p>
     *
     * <p>
     * This method returns <tt>null</tt> if the nickname has not been set.</p>
     *
     * @return The nickname for this <tt>RegisterQuery</tt>.
     * @since   JSO-0.12
     */
    public String getNickname();
    /**
     * <p>
     * Sets the nickname for this <tt>RegisterQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>nick</tt> may be <tt>null</tt> or "". Note that calling
     * this method always results in the inclusion of the "nickname" field.
     * Call {@link #clearNickname} to remove this field.</p>
     *
     * @param nick The new nickname.
     * @since   JSO-0.12
     */
    public void setNickname(String nick);
    /**
     * <p>
     * Removes the nickname from this <tt>RegisterQuery</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public void clearNickname();
    
    /**
     * <p>
     * Retrieves the full name for this <tt>RegisterQuery</tt>. This corresponds
     * to the field "name".</p>
     *
     * <p>
     * This method returns <tt>null</tt> if the full name has not been set.</p>
     *
     * @return The full name for this <tt>RegisterQuery</tt>.
     * @since   JSO-0.12
     */
    public String getFullName();
    /**
     * <p>
     * Sets the full name for this <tt>RegisterQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>name</tt> may be <tt>null</tt> or "". Note that calling
     * this method always results in the inclusion of the "fullname" field.
     * Call {@link #clearFullName} to remove this field.</p>
     *
     * @param name The new full name.
     * @since   JSO-0.12
     */
    public void setFullName(String name);
    /**
     * <p>
     * Removes the full name from this <tt>RegisterQuery</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public void clearFullName();
    
    /**
     * <p>
     * Retrieves the first name for this <tt>RegisterQuery</tt>. This
     * corresponds to the field "first".</p>
     *
     * <p>
     * This method returns <tt>null</tt> if the first name has not been set.</p>
     *
     * @return The first name for this <tt>RegisterQuery</tt>.
     * @since   JSO-0.12
     */
    public String getFirstName();
    /**
     * <p>
     * Sets the first name for this <tt>RegisterQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>first</tt> may be <tt>null</tt> or "". Note that calling
     * this method always results in the inclusion of the "first" field.
     * Call {@link #clearFirstName} to remove this field.</p>
     *
     * @param first The new first name.
     * @since   JSO-0.12
     */
    public void setFirstName(String first);
    /**
     * <p>
     * Removes the first name from this <tt>RegisterQuery</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public void clearFirstName();
    
    /**
     * <p>
     * Retrieves the last name for this <tt>RegisterQuery</tt>. This
     * corresponds to the field "last".</p>
     *
     * <p>
     * This method returns <tt>null</tt> if the last name has not been set.</p>
     *
     * @return The last name for this <tt>RegisterQuery</tt>.
     * @since   JSO-0.12
     */
    public String getLastName();
    /**
     * <p>
     * Sets the last name for this <tt>RegisterQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>last</tt> may be <tt>null</tt> or "". Note that calling
     * this method always results in the inclusion of the "last" field.
     * Call {@link #clearLastName} to remove this field.</p>
     *
     * @param last The new last name.
     * @since   JSO-0.12
     */
    public void setLastName(String last);
    /**
     * <p>
     * Removes the last name from this <tt>RegisterQuery</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public void clearLastName();
    
    /**
     * <p>
     * Retrieves the e-mail address for this <tt>RegisterQuery</tt>. This
     * corresponds to the field "email".</p>
     *
     * <p>
     * This method returns <tt>null</tt> if the e-mail has not been set.</p>
     *
     * @return The e-mail address for this <tt>RegisterQuery</tt>.
     * @since   JSO-0.12
     */
    public String getEMail();
    /**
     * <p>
     * Sets the e-mail address for this <tt>RegisterQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>email</tt> may be <tt>null</tt> or "". Note that calling
     * this method always results in the inclusion of the "email" field.
     * Call {@link #clearEMail} to remove this field.</p>
     *
     * @param email The new e-mail address.
     * @since   JSO-0.12
     */
    public void setEMail(String email);
    /**
     * <p>
     * Removes the e-mail address from this <tt>RegisterQuery</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public void clearEMail();
    
    /**
     * <p>
     * Retrieves the phone number for this <tt>RegisterQuery</tt>. This
     * corresponds to the field "phone".</p>
     *
     * <p>
     * This method returns <tt>null</tt> if the phone has not been set.</p>
     *
     * @return The phone number for this <tt>RegisterQuery</tt>.
     * @since   JSO-0.12
     */
    public String getPhone();
    /**
     * <p>
     * Sets the phone number for this <tt>RegisterQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>phone</tt> may be <tt>null</tt> or "". Note that calling
     * this method always results in the inclusion of the "phone" field.
     * Call {@link #clearPhone} to remove this field.</p>
     *
     * @param phone The new phone number.
     * @since   JSO-0.12
     */
    public void setPhone(String phone);
    /**
     * <p>
     * Removes the phone number from this <tt>RegisterQuery</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public void clearPhone();
    
    /**
     * <p>
     * Retrieves the URL for this <tt>RegisterQuery</tt>. This
     * corresponds to the field "url".</p>
     *
     * <p>
     * This method returns <tt>null</tt> if the URL has not been set.</p>
     *
     * @return The URL for this <tt>RegisterQuery</tt>.
     * @since   JSO-0.12
     */
    public URL getURL();
    /**
     * <p>
     * Sets the URL for this <tt>RegisterQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>url</tt> may be <tt>null</tt>. Note that calling
     * this method always results in the inclusion of the "url" field.
     * Call {@link #clearURL} to remove this field.</p>
     *
     * @param url The new URL.
     * @since   JSO-0.12
     */
    public void setURL(URL url);
    /**
     * <p>
     * Removes the URL from this <tt>RegisterQuery</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public void clearURL();
    
    /**
     * <p>
     * Retrieves the address for this <tt>RegisterQuery</tt>. This
     * corresponds to the field "address".</p>
     *
     * <p>
     * This method returns <tt>null</tt> if the address has not been set.</p>
     *
     * @return The address for this <tt>RegisterQuery</tt>.
     * @since   JSO-0.12
     */
    public String getAddress();
    /**
     * <p>
     * Sets the address for this <tt>RegisterQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>addr</tt> may be <tt>null</tt> or "". Note that calling
     * this method always results in the inclusion of the "address" field.
     * Call {@link #clearAddress} to remove this field.</p>
     *
     * @param addr The new address.
     * @since   JSO-0.12
     */
    public void setAddress(String addr);
    /**
     * <p>
     * Removes the address from this <tt>RegisterQuery</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public void clearAddress();
    
    /**
     * <p>
     * Retrieves the city for this <tt>RegisterQuery</tt>. This
     * corresponds to the field "city".</p>
     *
     * <p>
     * This method returns <tt>null</tt> if the city has not been set.</p>
     *
     * @return The city for this <tt>RegisterQuery</tt>.
     * @since   JSO-0.12
     */
    public String getCity();
    /**
     * <p>
     * Sets the city for this <tt>RegisterQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>city</tt> may be <tt>null</tt> or "". Note that calling
     * this method always results in the inclusion of the "city" field.
     * Call {@link #clearCity} to remove this field.</p>
     *
     * @param city The new city.
     * @since   JSO-0.12
     */
    public void setCity(String city);
    /**
     * <p>
     * Removes the city from this <tt>RegisterQuery</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public void clearCity();
    
    /**
     * <p>
     * Retrieves the state for this <tt>RegisterQuery</tt>. This
     * corresponds to the field "state".</p>
     *
     * <p>
     * This method returns <tt>null</tt> if the state has not been set.</p>
     *
     * @return The state for this <tt>RegisterQuery</tt>.
     * @since   JSO-0.12
     */
    public String getState();
    /**
     * <p>
     * Sets the state for this <tt>RegisterQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>st</tt> may be <tt>null</tt> or "". Note that calling
     * this method always results in the inclusion of the "state" field.
     * Call {@link #clearState} to remove this field.</p>
     *
     * @param st The new state.
     * @since   JSO-0.12
     */
    public void setState(String st);
    /**
     * <p>
     * Removes the state from this <tt>RegisterQuery</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public void clearState();
    
    /**
     * <p>
     * Retrieves the ZIP code for this <tt>RegisterQuery</tt>. This
     * corresponds to the field "zip".</p>
     *
     * <p>
     * This method returns <tt>null</tt> if the ZIP code has not been set.</p>
     *
     * @return The ZIP code for this <tt>RegisterQuery</tt>.
     * @since   JSO-0.12
     */
    public String getZIP();
    /**
     * <p>
     * Sets the ZIP code for this <tt>RegisterQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>zip</tt> may be <tt>null</tt> or "". Note that calling
     * this method always results in the inclusion of the "zip" field.
     * Call {@link #clearZIP} to remove this field.</p>
     *
     * @param zip The new ZIP code.
     * @since   JSO-0.12
     */
    public void setZIP(String zip);
    /**
     * <p>
     * Removes the ZIP code from this <tt>RegisterQuery</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public void clearZIP();
    
    /**
     * <p>
     * Retrieves the date for this <tt>RegisterQuery</tt>. This
     * corresponds to the field "date".</p>
     *
     * <p>
     * This method returns <tt>null</tt> if the date has not been set.</p>
     *
     * @return The date for this <tt>RegisterQuery</tt>.
     * @since   JSO-0.12
     */
    public String getDate();
    /**
     * <p>
     * Sets the date for this <tt>RegisterQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>date</tt> may be <tt>null</tt> or "". Note that calling
     * this method always results in the inclusion of the "date" field.
     * Call {@link #clearDate} to remove this field.</p>
     *
     * @param date The new date.
     * @since   JSO-0.12
     */
    public void setDate(String date);
    /**
     * <p>
     * Removes the date from this <tt>RegisterQuery</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public void clearDate();
    
    /**
     * <p>
     * Retrieves the miscellaneos information for this <tt>RegisterQuery</tt>.
     * This corresponds to the field "misc".</p>
     *
     * <p>
     * This method returns <tt>null</tt> if the miscellaneous information
     * has not been set.</p>
     *
     * @return The miscellaneous information for this <tt>RegisterQuery</tt>.
     * @since   JSO-0.12
     */
    public String getMisc();
    /**
     * <p>
     * Sets the miscellaneous information for this <tt>RegisterQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>misc</tt> may be <tt>null</tt> or "". Note that calling
     * this method always results in the inclusion of the "misc" field.
     * Call {@link #clearMisc} to remove this field.</p>
     *
     * @param misc The new miscellaneous information.
     * @since   JSO-0.12
     */
    public void setMisc(String misc);
    /**
     * <p>
     * Removes the miscellaneous information from this
     * <tt>RegisterQuery</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public void clearMisc();
    
    /**
     * <p>
     * Retrieves the text entry for this <tt>RegisterQuery</tt>. This
     * corresponds to the field "date".</p>
     *
     * <p>
     * This method returns <tt>null</tt> if the text entry has not been set.</p>
     *
     * @return The text entry for this <tt>RegisterQuery</tt>.
     * @since   JSO-0.12
     */
    public String getText();
    /**
     * <p>
     * Sets the text entry for this <tt>RegisterQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>text</tt> may be <tt>null</tt> or "". Note that calling
     * this method always results in the inclusion of the "text" field.
     * Call {@link #clearText} to remove this field.</p>
     *
     * @param text The new text entry.
     * @since   JSO-0.12
     */
    public void setText(String text);
    /**
     * <p>
     * Removes the text entry from this <tt>RegisterQuery</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public void clearText();
    
    /**
     * <p>
     * Retrieves the set of field names for this <code>RegisterQuery</code>.
     * The returned set reflects only those fields currently present.</p>
     *
     * <p>
     * "Fields" are any contained elements in the same namespace as
     * <code>RegisterQuery</code>, and does not include "instructions",
     * "registered", or "remove".</p>
     *
     * <p>
     * The value returned by this method is never <code>null</code>.</p>
     *
     * @return The set of field names present.
     * @since   JSO-0.4
     */
    public Set getFieldNames();
    /**
     * <p>
     * Retrieves the value for the given field in this
     * <code>RegisterQuery</code>.</p>
     *
     * <p>
     * The value of <code>name</code> cannot be <code>null</code> or "", or an
     * <code>IllegalArgumentException</code> is thrown.</p>
     *
     * <p>
     * The value returned by this method may be <code>null</code>, if field
     * is not present.</p>
     *
     * @param name The field name.
     * @return The field value.
     * @throws IllegalArgumentException if the parameter is invalid.
     * @since   JSO-0.4
     */
    public String getField(String name) throws IllegalArgumentException;
    /**
     * <p>
     * Sets the value for the given field in this <code>RegisterQuery</code>.
     * This method always results in the field being "set".  To remove a field,
     * use {@link #unsetField} instead.</p>
     *
     * <p>
     * The value of <code>name</code> cannot be <code>null</code> or "", or an
     * <code>IllegalArgumentException</code> is thrown.</p>
     *
     * @param name The field name.
     * @throws IllegalArgumentException if the parameter is invalid.
     * @since   JSO-0.4
     */
    public void setField(String name, String value) throws IllegalArgumentException;
    /**
     * <p>
     * Removes the field from this <code>RegisterQuery</code>.</p>
     *
     * <p>
     * The value of <code>name</code> cannot be <code>null</code> or "", or an
     * <code>IllegalArgumentException</code> is thrown.</p>
     *
     * @param name The field name.
     * @throws IllegalArgumentException if the parameter is invalid.
     * @since   JSO-0.4
     */
    public void unsetField(String name) throws IllegalArgumentException;
}