/*
 * SessionFeatureConsumer.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.x.core;


import java.util.Collections;
import java.util.Map;

import org.jaxen.SimpleVariableContext;

import org.saxpath.SAXPathException;

import org.jabberstudio.jso.InfoQuery;
import org.jabberstudio.jso.PacketRouter;
import org.jabberstudio.jso.StreamException;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamFeature;
import org.jabberstudio.jso.StreamFeatureset;
import org.jabberstudio.jso.event.PacketDispatcher;
import org.jabberstudio.jso.event.PacketEvent;
import org.jabberstudio.jso.features.AbstractFeatureConsumer;
import org.jabberstudio.jso.features.FeatureContext;
import org.jabberstudio.jso.util.IdentityGenerator;
import org.jabberstudio.jso.util.PacketException;
import org.jabberstudio.jso.util.XPathListener;

/**
 * <p>
 * <tt>FeatureConsumer</tt> implementation for consuming session
 * establishment.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public class SessionFeatureConsumer extends AbstractFeatureConsumer {
    //Inner-classes
    private class SessionWatcher extends XPathListener {
        //Constructors
        public SessionWatcher(PacketRouter router) throws SAXPathException {
            super(router.getDataFactory(), "app:iq[(@id=$id) and ((@type='result') or (@type='error'))]");
            
            Map     nss = new java.util.HashMap();
            nss.put("app", router.getDefaultNamespace());
            nss.put("auth", AuthQuery.NAMESPACE);
            
            setupNamespaces(nss);
            getXPath().setVariableContext(new SimpleVariableContext());
        }
        
        //Methods
        public void packetMatched(PacketEvent evt) {
            PacketDispatcher    dispatch = getFeatureContext().getDispatcher();
            InfoQuery           iq = (InfoQuery)evt.getData();
            SessionQuery        session;

            dispatch.removePacketListener(PacketEvent.RECEIVED, this);
            if (iq.getType() == InfoQuery.ERROR) {
                completeFailedConsumption(new PacketException(iq.getError()));
            } else {
                completeSucceededConsumption(false);
            }
        }
    }
    
    //Fields
    private boolean     _StrictNS;
    
    //Constructors
    /**
     * <p>
     * Creates a new <tt>SessionFeatureConsumer</tt>. By default, this
     * <tt>SessionFeatureConsumer</tt> operates in strict mode.</p>
     *
     * @since   JSO-0.12
     * @see #isStrict
     */
    public SessionFeatureConsumer() {
        this(true);
    }
    /**
     * <p>
     * Creates a new <tt>SessionFeatureConsumer</tt> with the given mode.</p>
     *
     * <p>
     * If the value of <tt>strict</tt> is <tt>true</tt>, this
     * <tt>SessionFeatureConsumer</tt> operates in strict mode.</p>
     *
     * @param strict <tt>true</tt> to operate in strict mode.
     * @since   JSO-0.12
     * @see #isStrict
     */
    public SessionFeatureConsumer(boolean strict) {
        _StrictNS = strict;
    }
    
    //Methods
    /**
     * <p>
     * Determines if this <tt>SessionFeatureConsumer</tt> is operating in
     * strict mode. Strict mode means this <tt>SessionFeatureConsumer</tt>
     * will only match if a reported feature exactly matches {@link
     * SessionQuery#NAME}. Otherwise, it matches if a reported feature has the
     * same local-name, ignoring namespace.</p>
     *
     * @return <tt>true</tt> if this <tt>SessionFeatureConsumer</tt> is in
     * strict mode.
     * @since   JSO-0.12
     * @deprecated This method is present only for compatibility reasons.
     * It may be removed at any time.
     */
    public boolean isStrict() {
        return _StrictNS;
    }
    /**
     * <p>
     * Sets the strict mode flag for this <tt>SessionFeatureConsumer</tt>.</p>
     *
     * @param strict <tt>true</tt> if this <tt>SessionFeatureConsumer</tt> is in
     * strict mode.
     * @since   JSO-0.12
     * @deprecated This method is present only for compatibility reasons.
     * It may be removed at any time.
     */
    public void setStrict(boolean strict) {
        _StrictNS = strict;
    }
    
    /**
     * <p>
     * Finds the <tt>StreamFeature</tt> in the given <tt>StreamFeatureset</tt>
     * that can be consumed. This implementation searches for the first
     * <tt>StreamElement</tt> with the NSI of {@link SessionQuery#NAME}
     * in <tt>fs</tt>.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if a consumable
     * <tt>StreamFeature</tt> could not be found.</p>
     *
     * <p>
     * The value of <tt>fs</tt> will never be <tt>null</tt>.</p>
     *
     * @param fs The <tt>StreamFeatureset</tt> to search in
     * @return The consumable <tt>StreamFeature</tt>.
     * @since   JSO-0.12
     */
    public StreamFeature findFeature(StreamFeatureset fs) {
        return (StreamFeature)(isStrict() ? fs.getFirstElement(SessionQuery.NAME) : fs.getFirstElement(SessionQuery.NAME.getLocalName()));
    }
    /**
     * <p>
     * Begins the session start process. This method registers a
     * <tt>PacketListener</tt> for session &lt;iq/&gt; packets, and sends the
     * session start request.</p>
     *
     * @throws IllegalStateException If the Session <tt>PacketListener</tt>
     * could not be registered.
     * @throws StreamException If there is a stream-level error
     * @since   JSO-0.12
     */
    protected void startConsumption() throws IllegalStateException, StreamException {
        FeatureContext      ctx = getFeatureContext();
        PacketRouter        router = ctx.getRouter();
        PacketDispatcher    dispatch = ctx.getDispatcher();
        StreamDataFactory   sdf = ctx.getRouter().getDataFactory();
        SessionWatcher      watch;
        InfoQuery           iq;
        SessionQuery        session;
        
        try {
            watch = new SessionWatcher(router);
            dispatch.addPacketListener(PacketEvent.RECEIVED, watch);
        } catch (SAXPathException spe) {
            throw new IllegalStateException("Could not register Session packet listener");
        }

        iq = (InfoQuery)sdf.createPacketNode(sdf.createNSI("iq", router.getDefaultNamespace()), InfoQuery.class);
        iq.setType(InfoQuery.SET);
        iq.setID(IdentityGenerator.generateGlobal("session"));

        session = (SessionQuery)iq.addElement(SessionQuery.NAME, SessionQuery.class);
        ((SimpleVariableContext)watch.getXPath().getVariableContext()).setVariableValue("id", iq.getID());

        router.send(iq);
    }
}