/*
 * ClassFunction.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.xpath;


import java.util.List;

import org.jaxen.Context;
import org.jaxen.FunctionCallException;
import org.jaxen.Navigator;
import org.jaxen.function.StringFunction;

/**
 * <p>
 * Class implementing the XPath function "class([object])".
 * This XPath function converts the given object (or first context node) into
 * a class instance.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public class ClassFunction extends AbstractFunction {
    //Constructors
    /**
     * <p>
     * Creates a new <tt>ClassFunction</tt> object.</p>
     *
     * @since   JSO-0.12
     */
    public ClassFunction() {
    }
    
    /**
     * <p>
     * Retrieves the "typical" function name for this
     * <tt>ClassFunction</tt>.</p>
     *
     * <p>
     * The value returned by this method is always "class".</p>
     *
     * @return The "typical" function name.
     * @since   JSO-0.12
     */
    public String getFunctionName() {
        return "class";
    }
    
    /**
     * <p>
     * Called during XPath evaluation to convert argument list into a
     * <tt>Class</tt>.</p>
     *
     * <p>
     * The value of <tt>ctx</tt> should not be <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>args</tt> should not be <tt>null</tt>. If <tt>args</tt>
     * is empty, the context node-set is used. If <tt>args</tt> contains more
     * than one value, a <tt>FunctionCallException</tt> is thrown.</p>
     *
     * @param ctx The Jaxen <tt>Context</tt> in use.
     * @param args The list of arguments to evaluate.
     * @return The <tt>Class</tt> representing the given argument list or
     * context node-set.
     * @throws FunctionCallException If the evaluation fails.
     * @since   JSO-0.12
     */
    public Object call(Context ctx, List args) throws FunctionCallException {
        Navigator   nav = ctx.getNavigator();
        Object      eval;
        int         size = args.size();
        
        if (size > 1)
            throw new FunctionCallException("class() requires zero or one argument");

        eval = (size == 0) ? ctx.getNodeSet() : args.get(0);
        
        return evaluate(eval, nav);
    }
    /**
     * <p>
     * Converts the given argument into a <tt>Class</tt> object.</p>
     *
     * <p>
     * The value of <tt>arg</tt> must either be a <tt>Class</tt> instance or
     * represent the full name of a valid class, or an
     * <tt>FunctionCallException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>nav</tt> cannot be <tt>null</tt>, or a
     * <tt>NullPointerException</tt> is thrown.</p>
     *
     * @param arg The object to convert.
     * @param nav The Jaxen <tt>Navigator</tt> to use.
     * @return The class represented by <tt>arg</tt>.
     * @throws FunctionCallException If the comparison cannot be made.
     * @throws NullPointerException If any required parameters are
     * <tt>null</tt>.
     * @since   JSO-0.12
     */
    public static Class evaluate(Object arg, Navigator nav) throws FunctionCallException {
        Class       iface = null;
        
        if (arg instanceof Class) {
            iface = (Class)arg;
        } else {
            try {
                iface = Class.forName(StringFunction.evaluate(arg, nav));
            } catch (Exception e) {
                throw new FunctionCallException("Could not convert object to Class", e);
            }
        }
        
        return iface;
    }
}