/*
 * EqualsJidFunction.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.xpath;


import java.util.List;

import org.jaxen.Context;
import org.jaxen.FunctionCallException;
import org.jaxen.Navigator;

import org.jabberstudio.jso.JID;

/**
 * <p>
 * Class implementing the XPath function "jid-equals(object, object)". This
 * XPath function attempts to compare (for equality) the first JID object to
 * the second, first converting both to <tt>JID</tt> instances.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public class EqualsJidFunction extends AbstractFunction {
    //Constructors
    /**
     * <p>
     * Creates a new <tt>EqualsJidFunction</tt> object.</p>
     *
     * @since   JSO-0.12
     */
    public EqualsJidFunction() {
    }
    
    //Methods
    /**
     * <p>
     * Retrieves the "typical" function name for this
     * <tt>EqualsJidFunction</tt>.</p>
     *
     * <p>
     * The value returned by this method is always "jid-equals".</p>
     *
     * @return The "typical" function name.
     */
    public String getFunctionName() {
        return "jid-equals";
    }
    
    /**
     * <p>
     * Called during XPath evaluation to match two arguments using {@link
     * JID#equals(JID)}. The arguments are first converted to <tt>JID</tt>
     * instances.</p>
     *
     * <p>
     * The value of <tt>ctx</tt> should not be <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>args</tt> should not be <tt>null</tt>. If <tt>args</tt>
     * is empty or contains more than two arguements, a
     * <tt>FunctionCallException</tt> is thrown.</p>
     *
     * @param ctx The Jaxen <tt>Context</tt> in use.
     * @param args The list of arguments to evaluate.
     * @return A <tt>Boolean</tt> value determining if the two "JIDs" are equal.
     * @throws FunctionCallException If the evaluation fails.
     * @since   JSO-0.12
     */
    public Object call(Context ctx, List args) throws FunctionCallException {
        Navigator   nav = ctx.getNavigator();
        int         size = args.size();
        
        if ((size == 0) || (size > 2))
            throw new FunctionCallException("jid() requires two arguments");

        return evaluate(args.get(0), args.get(1), nav);
    }
    
    /**
     * <p>
     * Compares the two arguments using the {@link JID#equals(JID)} method.
     * Both arguments are first converted to <tt>JID</tt> instances. The first
     * argument is used as the "source" to compare against, while the second
     * argument is the target being compared.</p>
     *
     * <p>
     * The value of <tt>arg1</tt> must evaluate to a valid <tt>JID</tt>, or a
     * <tt>FunctionCallException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>arg2</tt> must evaluate to a valid <tt>JID</tt>, or a
     * <tt>FunctionCallException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>nav</tt> cannot be <tt>null</tt> or an
     * <tt>NullPointerException</tt> is thrown.</p>
     *
     * @param arg1 The "source" JID.
     * @param arg2 The "target" JID.
     * @param nav The Jaxen <tt>Navigator</tt> to use.
     * @return <tt>true</tt> (as a <tt>Boolean</tt> object) if the "target"
     * equals the "source", and <tt>false</tt> (as a <tt>Boolean</tt> object)
     * otherwise.
     * @throws FunctionCallException If the argument cannot be converted to a
     * <tt>JID</tt>.
     * @throws NullPointerException If any required parameters are
     * <tt>null</tt>.
     * @since   JSO-0.12
     */
    public static Boolean evaluate(Object arg1, Object arg2, Navigator nav) throws FunctionCallException {
        JID         source, target;
        Boolean     retval = null;
        
        try {
            source = JidFunction.evaluate(arg1, nav);
            if (source == null)
                throw new FunctionCallException("first argument is not a valid JID");
            
            target = JidFunction.evaluate(arg2, nav);
            if (target == null)
                throw new FunctionCallException("second argument is not a valid JID");
            
            retval = Boolean.valueOf(source.equals(target));
        } catch (IllegalArgumentException iae) {
            throw new FunctionCallException("Cannot evaluate arguments");
        }
        
        return retval;
    }
}