/*
 * ImplsClassFunction.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.xpath;


import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.jaxen.Context;
import org.jaxen.FunctionCallException;
import org.jaxen.Navigator;

/**
 * <p>
 * Class implementing the XPath function "implements-class(class [, object...])".
 * This XPath function determines if the given object, list of objects, or
 * context nodeset implements the given class.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public class ImplsClassFunction extends AbstractFunction {
    //Constructors
    /**
     * <p>
     * Creates a new <tt>ImplsClassFunction</tt> object.</p>
     *
     * @since   JSO-0.12
     */
    public ImplsClassFunction() {
    }

    //Methods
    /**
     * <p>
     * Retrieves the "typical" function name for this
     * <tt>ImplsClassFunction</tt>.</p>
     *
     * <p>
     * The value returned by this method is always "implements-class".</p>
     *
     * @return The "typical" function name.
     * @since   JSO-0.12
     */
    public String getFunctionName() {
        return "implements-class";
    }
    
    /**
     * <p>
     * Called during XPath evaluation to determine if the arguments (after the
     * first) or the context node-set implement a given class.</p>
     *
     * <p>
     * The value of <tt>ctx</tt> should not be <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>args</tt> may not be <tt>null</tt>, or a
     * <tt>FunctionCallException</tt> is thrown. If <tt>args</tt> contains only
     * one element, the context node-set is used for evaluation. If the first
     * element of <tt>args</tt> does not represent a valid class, a
     * <tt>FunctionCallException</tt> is thrown.</p>
     *
     * @param ctx The Jaxen <tt>Context</tt> in use.
     * @param args The list of arguments to evaluate.
     * @return The <tt>Boolean</tt> result of the determination.
     * @throws FunctionCallException If the evaluation fails.
     * @since   JSO-0.12
     */
    public Object call(Context ctx, List args) throws FunctionCallException {
        Navigator   nav = ctx.getNavigator();
        Class       iface;
        int         size = args.size();

        //Validate input
        if (size < 1)
            throw new FunctionCallException("implements-class() requires one or more arguments");

        iface = ClassFunction.evaluate(args.get(0), nav);
        args = ((size == 1) ? ctx.getNodeSet() : asFlatList(args.subList(1, size)));
        
        return evaluate(iface, args, nav);
    }
    /**
     * <p>
     * Determines if the given list of arguments implements the given
     * class.</p>
     *
     * <p>
     * The value of <tt>iface</tt> cannot be <tt>null</tt>, or a
     * <tt>FunctionCallException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>args</tt> may be <tt>null</tt>, in which case an empty
     * list is used.</p>
     *
     * <p>
     * The value of <tt>nav</tt> cannot be <tt>null</tt> or a
     * <tt>NullPointerException</tt> is thrown.</p>
     *
     * @param iface The class to be implemented.
     * @param args The list of objects to compare.
     * @param nav The Jaxen <tt>Navigator</tt> to use.
     * @return <tt>true</tt> (as a <tt>Boolean</tt>) if all objects in
     * <tt>args</tt> are an instance of <tt>iface</tt>.
     * @throws FunctionCallException If the comparison cannot be made.
     * @throws NullPointerException If any required parameters are
     * <tt>null</tt>.
     * @since   JSO-0.12
     */
    public static Boolean evaluate(Class iface, List args, Navigator nav) throws FunctionCallException {
        Iterator        itr;
        boolean         result;
        
        //Validate input
        if (iface == null)
            throw new FunctionCallException("Class cannot be null");
        if (args == null)
            args = java.util.Collections.EMPTY_LIST;
        itr = args.iterator();
        result = !args.isEmpty();

        //Let's compare!
        while (result && itr.hasNext()) {
            result = (result && iface.isInstance(itr.next()));
        }
        
        return Boolean.valueOf(result);
    }
    
    private List asFlatList(List in) {
        Iterator    itr = in.iterator();
        Object      test;
        List        out = new java.util.ArrayList();
        
        while (itr.hasNext()) {
            test = itr.next();
            
            if      (test instanceof Collection)
                out.addAll((Collection)test);
            else if (test instanceof Map)
                out.addAll(((Map)test).values());
            else
                out.add(test);
        }
        
        return out;
    }
}